#include <pspkernel.h>
#include <pspctrl.h>
#include <pspdebug.h>
#include <pspsdk.h>
#include <time.h>
#include <psprtc.h>

#include <string.h>
#include <pspreg.h>

#include "fileOperation.h"

PSP_MODULE_INFO("homebrewSorter", PSP_MODULE_USER, 1, 0);
PSP_MAIN_THREAD_ATTR(THREAD_ATTR_USER);
PSP_HEAP_SIZE_KB(2*1024);

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// define:
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define RGB(r, g, b) ((b << 16) | (g << 8) | r)
#define DARKBLUE RGB(0, 104, 139)
#define LIGHTBLUE RGB(0, 178, 238)
#define BLACK RGB(0, 0, 0)
#define WHITE RGB(255, 255, 255)
#define RED RGB(255, 0, 0)
#define YELLOW RGB(255, 255, 0)

#define ANALOG_SENS 80
#define VERSION "v01"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Globals:
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int runningFlag = 1;
struct category CATlist[MAX_CAT];
struct homebrew HBlist[MAX_HB];
int CATcount = 0;//add by mascii
int HBcount = 0;

unsigned int but;
int cfmbtn = PSP_CTRL_CROSS;
int cnclbtn = PSP_CTRL_CIRCLE;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Callbacks:
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/* Exit callback */
int exit_callback(int arg1, int arg2, void *common) {
    runningFlag = 0;
    return 0;
}

/* Callback thread */
int CallbackThread(SceSize args, void *argp) {
    int cbid;
    cbid = sceKernelCreateCallback("Exit Callback", exit_callback, NULL);
    sceKernelRegisterExitCallback(cbid);
    sceKernelSleepThreadCB();
    return 0;
}

/* Sets up the callback thread and returns its thread id */
int SetupCallbacks(void) {
    int thid = 0;
    thid = sceKernelCreateThread("update_thread", CallbackThread, 0x11, 0xFA0, PSP_THREAD_ATTR_USER, 0);
    if(thid >= 0)
        sceKernelStartThread(thid, 0, 0);
    return thid;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Get Registry Value
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int get_registry_value(const char *dir, const char *name, unsigned int *val)
{
	int ret = 0;
	struct RegParam reg;
	REGHANDLE h;

	memset(&reg, 0, sizeof(reg));
	reg.regtype = 1;
	reg.namelen = strlen("/system");
	reg.unk2 = 1;
	reg.unk3 = 1;
	strcpy(reg.name, "/system");
	if(sceRegOpenRegistry(&reg, 2, &h) == 0)
	{
		REGHANDLE hd;
		if(!sceRegOpenCategory(h, dir, 2, &hd))
		{
			REGHANDLE hk;
			unsigned int type, size;

			if(!sceRegGetKeyInfo(hd, name, &hk, &type, &size))
			{
				if(!sceRegGetKeyValue(hd, hk, val, 4))
				{
					ret = 1;
					sceRegFlushCategory(hd);
				}
			}
			sceRegCloseCategory(hd);
		}
		sceRegFlushRegistry(h);
		sceRegCloseRegistry(h);
	}

	return ret;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Save list order:
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int saveList(){
    int i;
    
	pspDebugScreenSetBackColor(BLACK);
    pspDebugScreenSetTextColor(RED);
    for (i=10; i<15; i++){
    	pspDebugScreenSetXY(22, i);
    	pspDebugScreenPrintf("%-25.25s", "");
    }
	pspDebugScreenSetXY(24, 12);
	pspDebugScreenPrintf("Saving list order...");
    saveHBlist(HBlist, HBcount);
    return 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Draw toolbars:
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void drawToolbars(){
//	pspDebugScreenSetBackColor(DARKBLUE);
    pspDebugScreenSetTextColor(RED);//(WHITE);
	pspDebugScreenSetXY(0, 0);
	pspDebugScreenPrintf("%-67.67s", "Homebrew Sorter for GCR by Sakya. Modified by Hide and mascii.");
	pspDebugScreenSetXY(0, 33);
	pspDebugScreenPrintf("%-67.67s", "");

}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Draw Current Time:
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void drawCurrentTime(){

	pspDebugScreenSetTextColor(RED);//(WHITE);

	//Current time:
/*
	struct tm * ptm;
	time_t mytime;
	time(&mytime);
	ptm = localtime(&mytime);
*/
	pspTime rtime;
	sceRtcGetCurrentClockLocalTime(&rtime);
	pspDebugScreenSetXY(51, 33);

	if (!but)
	pspDebugScreenPrintf("%4.4d/%2.2d/%2.2d %2.2d:%2.2d",rtime.year, rtime.month, rtime.day, rtime.hour,rtime.minutes);
	else
	pspDebugScreenPrintf("%2.2d/%2.2d/%4.4d %2.2d:%2.2d",rtime.day, rtime.month, rtime.year, rtime.hour,rtime.minutes);

}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Main menu:
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int mainMenu(){
    SceCtrlData pad;
    int start = 4;
    int first = 0;
    int total = HBcount;
    int visible = 23;
    int selected = 0;
    int i = 0;
    int times = 0;
    int updateInfo = 10;

	pspDebugScreenInit();

        drawToolbars();

        pspDebugScreenSetXY(0, 1);
        pspDebugScreenSetTextColor(RED);
        pspDebugScreenSetBackColor(BLACK);
        pspDebugScreenPrintf("Total homebrews found: %i", HBcount);
        pspDebugScreenSetTextColor(WHITE);
        pspDebugScreenSetBackColor(BLACK);
        pspDebugScreenSetXY(0, 30);
        pspDebugScreenPrintf("%-67.67s","Press []+UP to move homebrew UP / []+DOWN to move homebrew DOWN.");
        pspDebugScreenSetXY(0, 31);
        pspDebugScreenPrintf("%-67.67s","Press L to move Bottom / R to move Top.");
        pspDebugScreenSetXY(0, 32);
        if (!but)
        pspDebugScreenPrintf("%-67.67s","Press START to save list / X to return to Categories Menu");
        else
        pspDebugScreenPrintf("%-67.67s","Press START to save list / O to return to Categories Menu");

        while (runningFlag){

        if (updateInfo >= 10){

        drawCurrentTime();

        //Draw menu:
        for (i=first; i<=first+visible; i++){
            if (i == selected){
                pspDebugScreenSetXY(0, 29);
                pspDebugScreenSetTextColor(YELLOW);
                pspDebugScreenSetBackColor(BLACK);

                if (HBcount != 0)
                pspDebugScreenPrintf("Path: %-60.60s", HBlist[i].path);
                else
                pspDebugScreenPrintf("Path: %-60.60s", "");

                pspDebugScreenSetTextColor(RED);
            }else
		pspDebugScreenSetTextColor(WHITE);

                pspDebugScreenSetBackColor(BLACK);//(DARKBLUE);

            pspDebugScreenSetXY(0, start + i - first);
            if (i < total){
                pspDebugScreenPrintf("%-67.67s", HBlist[i].name);
            }else{
                pspDebugScreenPrintf("%-67.67s", "");
            }
        }

        updateInfo = 0;
        }
        updateInfo++;

        if(!times){
            sceKernelDelayThread(300000);
            times = 1;
        }

        sceCtrlReadBufferPositive(&pad, 1);
        if (pad.Buttons & PSP_CTRL_SQUARE) {
            if (pad.Buttons & PSP_CTRL_DOWN) {
                if (selected < HBcount - 1){
                    moveHBdown(selected, HBlist);
                    if (++selected > first + visible)
                        first++;
                }
                updateInfo = 10;
                sceKernelDelayThread(200000);
            }else if (pad.Buttons & PSP_CTRL_UP){
                if (selected > 0){
                    moveHBup(selected, HBlist);
                    if (--selected < first)
                        first--;
                }
                updateInfo = 10;
                sceKernelDelayThread(200000);
            }else if (pad.Ly > 128 + ANALOG_SENS && !(pad.Buttons & PSP_CTRL_HOLD)) {
                if (selected < HBcount - 1){
                    moveHBdown(selected, HBlist);
                    if (++selected > first + visible)
                        first++;
                }
                updateInfo = 10;
            }else if (pad.Ly < 128 - ANALOG_SENS && !(pad.Buttons & PSP_CTRL_HOLD)){
                if (selected > 0){
                    moveHBup(selected, HBlist);
                    if (--selected < first)
                        first--;
                }
                updateInfo = 10;
            }
        }else if (pad.Buttons & PSP_CTRL_DOWN) {
            if (selected < total - 1){
                if (++selected > first + visible)
                    first++;
            }
            updateInfo = 10;
            sceKernelDelayThread(100000);
        }else if (pad.Buttons & PSP_CTRL_UP){
            if (selected > 0){
                if (--selected < first)
                    first--;
            }
            updateInfo = 10;
            sceKernelDelayThread(100000);
        }else if (pad.Ly > 128 + ANALOG_SENS && !(pad.Buttons & PSP_CTRL_HOLD)) {
            if (selected < total - 1){
                if (++selected > first + visible)
                    first++;
            }
            updateInfo = 10;
        }else if (pad.Ly < 128 - ANALOG_SENS && !(pad.Buttons & PSP_CTRL_HOLD)){
            if (selected > 0){
                if (--selected < first)
                    first--;
            }
            updateInfo = 10;
        }else if (pad.Buttons & PSP_CTRL_RTRIGGER){
            if (selected > 0){
                moveHBTop(selected, HBlist);
				selected=first;
            }
            updateInfo = 10;
            sceKernelDelayThread(200000);
        }else if (pad.Buttons & PSP_CTRL_LTRIGGER){
            if (selected < HBcount -1){
                moveHBBottom(selected, HBcount, HBlist);
				selected=HBcount -1;
            }
            updateInfo = 10;
            sceKernelDelayThread(200000);
        }else if (pad.Buttons & PSP_CTRL_START){
            saveList();
            updateInfo = 10;
            sceKernelDelayThread(200000);
        }else if (pad.Buttons & cnclbtn){
            break;
        }
    }
    return 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Categories menu:
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int SelectMenu(){
    SceCtrlData pad;
    int start = 4;
    int first = 0;
    int total = CATcount;
    int visible = 23;
    int selected = 0;
    int i = 0;
    int refresh = 1;
    int updateInfo = 10;

	pspDebugScreenInit();

	drawToolbars();

	while (runningFlag){

        if(refresh){

        pspDebugScreenSetXY(0, 1);
        pspDebugScreenSetTextColor(RED);
        pspDebugScreenSetBackColor(BLACK);
        pspDebugScreenPrintf("Total categories found: %i", CATcount);
        pspDebugScreenSetTextColor(WHITE);
        pspDebugScreenSetBackColor(BLACK);
        pspDebugScreenSetXY(0, 30);
        if (!but)
        pspDebugScreenPrintf("%-67.67s","Press O to open selected category.");
        else
        pspDebugScreenPrintf("%-67.67s","Press X to open selected category.");

        pspDebugScreenSetXY(0, 31);
        if (!but)
        pspDebugScreenPrintf("%-67.67s","Press X to exit.");
        else
        pspDebugScreenPrintf("%-67.67s","Press O to exit.");

        pspDebugScreenSetXY(0, 32);
        pspDebugScreenPrintf("%-67.67s","");

        }

        if (updateInfo >= 10){

        drawCurrentTime();

        //Draw menu:
        for (i=first; i<=first+visible; i++){
            if (i == selected){
                pspDebugScreenSetXY(0, 29);
                pspDebugScreenSetTextColor(YELLOW);
                pspDebugScreenSetBackColor(BLACK);
                pspDebugScreenPrintf("Category: %-56.56s", CATlist[i].name);
                pspDebugScreenSetTextColor(RED);
            }else
		pspDebugScreenSetTextColor(WHITE);

                pspDebugScreenSetBackColor(BLACK);//(DARKBLUE);

            pspDebugScreenSetXY(0, start + i - first);
            if (i < total){
                pspDebugScreenPrintf("%-67.67s", CATlist[i].name);
            }else{
                pspDebugScreenPrintf("%-67.67s", "");
            }
        }

        updateInfo = 0;
        }
        updateInfo++;

        if(refresh){
            sceKernelDelayThread(300000);
            refresh = 0;
        }

        sceCtrlReadBufferPositive(&pad, 1);
        if (pad.Buttons & PSP_CTRL_DOWN) {
            if (selected < total - 1){
                if (++selected > first + visible)
                    first++;
            }
            updateInfo = 10;
            sceKernelDelayThread(100000);
        }else if (pad.Buttons & PSP_CTRL_UP){
            if (selected > 0){
                if (--selected < first)
                    first--;
            }
            updateInfo = 10;
            sceKernelDelayThread(100000);
        }else if (pad.Ly > 128 + ANALOG_SENS && !(pad.Buttons & PSP_CTRL_HOLD)) {
            if (selected < total - 1){
                if (++selected > first + visible)
                    first++;
            }
            updateInfo = 10;
        }else if (pad.Ly < 128 - ANALOG_SENS && !(pad.Buttons & PSP_CTRL_HOLD)){
            if (selected > 0){
                if (--selected < first)
                    first--;
            }
            updateInfo = 10;
        }else if (pad.Buttons & cfmbtn){
            HBcount = getHBList(HBlist, CATlist[selected].name);
            mainMenu();
            refresh = 1;
            updateInfo = 10;
        }else if (pad.Buttons & cnclbtn){
            sceKernelExitGame();
        }
    }
    return 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Main:
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int main(){
	pspDebugScreenInit();
	SetupCallbacks();

    tzset();
    sceCtrlSetSamplingCycle(0);
    sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);

    get_registry_value("/CONFIG/SYSTEM/XMB", "button_assign", &but);
    if (!but){
        cfmbtn = PSP_CTRL_CIRCLE;
        cnclbtn = PSP_CTRL_CROSS;
    }

//    HBcount = getHBList(HBlist);
//    mainMenu();
    CATcount = getCATList(CATlist);
    SelectMenu();

	sceKernelExitGame();
    return 0;

}
